package http

import (
	"net/http"

	"git.javil.eu/jacob1123/budgeteer"

	"gopkg.in/gin-gonic/gin.v1"
)

// Handler handles incoming requests
type Handler struct {
	UserService budgeteer.UserService
}

func (h *Handler) Serve() {

	router := gin.Default()

	router.LoadHTMLGlob("./templates/*")
	router.Static("/static", "./static")

	router.GET("/", func(c *gin.Context) { c.HTML(http.StatusOK, "index", nil) })
	router.GET("/login", login)
	api := router.Group("/api/v1")
	{
		api.GET("/logout", logout)
		api.GET("/login", func(c *gin.Context) {
			c.Redirect(http.StatusPermanentRedirect, "/login")
		})
		api.POST("/login", loginPost)

		// Unauthenticated routes
		api.GET("/check", func(c *gin.Context) {
			c.String(http.StatusOK, "Accessible")
		})
		api.GET("/hello", func(c *gin.Context) {
			c.String(http.StatusOK, "Hello, World!")
		})

		// Restricted group
		r := api.Group("/restricted")
		{
			//r.Use(middleware.JWT([]byte(secret)))
			r.GET("", restricted)
		}
	}

	router.Run(":1323")
}

func restricted(c *gin.Context) {
	claims, ok := verifyLogin(c)
	if !ok {
		c.Redirect(http.StatusTemporaryRedirect, "/login")
		return
	}

	name := claims["name"].(string)
	c.String(http.StatusOK, "Welcome "+name+"!")
}

func verifyLogin(c *gin.Context) error {
	tokenString, err := c.Cookie(authCookie)
	if err != nil {
		return nil, err
	}

}

func login(c *gin.Context) {
	if _, ok := verifyLogin(c); ok {
		c.Redirect(http.StatusTemporaryRedirect, "/api/v1/hello")
		return
	}

	c.HTML(http.StatusOK, "login", nil)
}

func logout(c *gin.Context) {
	clearLogin(c)
}

func loginPost(c *gin.Context) {
	username, _ := c.GetPostForm("username")
	password, _ := c.GetPostForm("password")

	if username != "jan" || password != "passwort" {
		c.AbortWithStatus(http.StatusUnauthorized)
		return
	}

	loginSuccess(c, username, "Jan Bader")
}
